/* ------------------------------------------------------------------------- */
/*!
 *  \file       atlas_generator.h
 *  \brief      アトラスジェネレータ
 *  \date       Since: June 7, 2021. 17:15:07 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_ATLAS_GENERATOR_H_1623053707
#define INCGUARD_ATLAS_GENERATOR_H_1623053707

#include <filesystem>
#include <algorithm>
#include <list>
#include <vector>

#include "bitmap.h"
#include "image_file_info.h"
#include "rectangle.h"

namespace texatlas
{
//! アトラスジェネレータ
class AtlasGenerator
{
public:
    //! コンストラクタ
    AtlasGenerator(bool isVerbose) noexcept
        : _targetList()
        , _bitmapArray()
        , _locations()
        , _hashSeed(0)
        , _isVerbose(isVerbose)
    {}

    size_t CreateTargetList(const std::vector<std::filesystem::path> &targetFiles, uint32_t hashSeed) noexcept;
    void PrintTargetList() const noexcept;
    size_t CheckCollidingHash() const noexcept;

    bool Generate(uint32_t width, uint32_t height, uint32_t margin, const char *outputName) noexcept;

    bool Save(const std::filesystem::path &outputPath, const std::filesystem::path &name) noexcept;

private:
    bool SaveLocatonToJson(const std::filesystem::path &outputPath, const std::filesystem::path &name) const noexcept;
    bool SaveLocatonToBinary(const std::filesystem::path &outputPath, const std::filesystem::path &name) const noexcept;

    std::list<ImageFileInfo> _targetList;
    std::vector<Bitmap> _bitmapArray;

    //! 配置情報
    struct Location
    {
        std::string name;       //!< 名前
        uint32_t nameHash;      //!< 名前のハッシュ値
        uint32_t index;         //!< テクスチャのインデックス
        uint32_t x;             //!< X座標
        uint32_t y;             //!< Y座標
        uint32_t width;         //!< 幅
        uint32_t height;        //!< 高さ

        //! コンストラクタ
        Location() noexcept
            : name()
            , nameHash(0)
            , index(0)
            , x(0)
            , y(0)
            , width(0)
            , height(0)
        {}
    };
    std::list<Location> _locations;

    uint32_t _hashSeed;
    bool _isVerbose;
};
}   // namespace

#endif	// INCGUARD_ATLAS_GENERATOR_H_1623053707

// vim: et ts=4 sw=4 sts=4
