#!/usr/bin/env bash

TARGET_DIR=manpages


# 更新
function update()
{
    find ${TARGET_DIR} -type f | grep "\.md$" | while read target
    do
        noext=${target%\.md}

        echo "${target} -> ${noext}"
        pandoc -s -t man ${target} -o ${noext}

        gzip ${noext}
        mv ${noext}.gz ${noext}

        echo "${target} -> ${noext}.html"
        pandoc -s -t html ${target} -o ${noext}.html
    done
}


# インストール
function install()
{
    if [ $# -eq 0 ]; then
        echo error
        exit -1
    fi

    find ${TARGET_DIR} -type f | grep "\.[0-9]$" | while read target
    do
        src=${target#${TARGET_DIR}/}
        dir=${src%/*}
        dest=$1/${dir}

        echo "copy ${target} -> ${dest}"
        mkdir -p ${dest}
        cp ${target} ${dest}
    done
}


# アンインストール
function uninstall()
{
    if [ $# -eq 0 ]; then
        echo error
        exit -1
    fi

    find ${TARGET_DIR} -type f | grep "\.[0-9]$" | while read target
    do
        src=${target#${TARGET_DIR}/}
        rmfile=$1/${src}

        echo "remove ${rmfile}"
        rm -f ${rmfile}
    done
}


case $1 in
    "update") update ;;
    "install") install $2;;
    "uninstall") uninstall $2;;
    *) exit -1 ;;
esac

exit 0
