// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       csv_parser.h
 *  \brief      CSVパーサークラス
 *  \date       Since: October 17, 2018. 15:32:20 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_CSV_PARSER_H_1539757940
#define INCGUARD_CSV_PARSER_H_1539757940

#include <filesystem>
#include <string>
#include <vector>

#include <mtk/text_stream.h>

namespace MTK
{
//! CSVパーサークラス
class CSVParser
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    CSVParser() noexcept
        : _csvFieldArray()
        , _columnCount(0)
        , _recordCount(0)
    {
    }

    bool Load(const std::filesystem::path &path, TextEncoding encoding = TextEncoding::Auto) noexcept;

    std::string GetField(int record, int column) const noexcept;


    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      レコードのフィールド数を取得
     *  \return     レコードのフィールド数
     */
    /* ------------------------------------------------------------------------- */
    constexpr int GetFieldCount() const noexcept
    {
        return _columnCount;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      レコードの数を取得
     *  \return     レコード数
     */
    /* ------------------------------------------------------------------------- */
    constexpr int GetRecordCount() const noexcept
    {
        return _recordCount;
    }

private:
    bool ParseCSV(TextStream &stream) noexcept;
    bool ParseEnclosedField(
        TextStream &stream,
        std::string &csvField,
        bool &isTerminatedRecord,
        bool &isTerminatedParse
    ) noexcept;


    std::vector<std::string> _csvFieldArray;
    int _columnCount;
    int _recordCount;
};
}   // namespace MTK

#endif  // INCGUARD_CSV_PARSER_H_1539757940

// vim: et ts=4 sw=4 sts=4
