// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       file_pointer.h
 *  \brief      ファイルポインタクラス
 *  \date       Since: January 23, 2023. 16:12:29 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_FILE_POINTER_H_1674457949
#define INCGUARD_FILE_POINTER_H_1674457949

#include <filesystem>
#include <stdio.h>

namespace MTK::File
{

class Pointer
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    constexpr Pointer() noexcept
        : _fp(nullptr)
    {
    }

    Pointer(const std::filesystem::path &path, const char *mode) noexcept;

    ~Pointer() noexcept;

    bool Open(const std::filesystem::path &path, const char *mode) noexcept;
    void Close() noexcept;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ファイルポインタがオープン済みかを取得
     *  \retval     true    オープン済み
     *  \retval     false   オープンしていない
     */
    /* ------------------------------------------------------------------------- */
    constexpr bool IsOpened() const noexcept
    {
        return _fp != nullptr;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      bool型へのキャスト
     *  \retval     true    オープン済み
     *  \retval     false   オープンしていない
     */
    /* ------------------------------------------------------------------------- */
    explicit constexpr operator bool() const noexcept
    {
        return IsOpened();
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      否定演算子のオーバーロード
     *  \retval     true    オープンしていない
     *  \retval     false   オープン済み
     */
    /* ------------------------------------------------------------------------- */
    constexpr bool operator!() const noexcept
    {
        return !IsOpened();
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ファイルポインタへのキャスト
     *  \return     ファイルポインタ
     */
    /* ------------------------------------------------------------------------- */
    constexpr operator FILE*() const noexcept
    {
        return _fp;
    }

private:
    FILE *_fp;
};

}   // namespace MTK::File

#endif	// INCGUARD_FILE_POINTER_H_1674457949

// vim: et ts=4 sw=4 sts=4
