// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       text_stream.h
 *  \brief      テキストストリームクラス
 *  \date       Since: October 4, 2018. 2:13:19 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_TEXT_STREAM_H_1538586799
#define INCGUARD_TEXT_STREAM_H_1538586799

#include <filesystem>
#include <memory>
#include <stddef.h>

#include <mtk/text_character.h>
#include <mtk/text_def.h>

namespace MTK
{
//! テキストストリームクラス
class TextStream
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    constexpr TextStream() noexcept
        : _buffer(nullptr)
        , _size(0)
        , _encoding(TextEncoding::Auto)
        , _offset(0)
        , _existBOM(false)
        , _loadBuffer(nullptr)
    {
    }

    TextStream(const uint8_t *buffer, size_t size, TextEncoding encoding = TextEncoding::Auto, bool existBOM = false) noexcept;

    void Set(const uint8_t *buffer, size_t size, TextEncoding encoding = TextEncoding::Auto, bool existBOM = false) noexcept;
    bool Load(const std::filesystem::path &path, TextEncoding encoding = TextEncoding::Auto) noexcept;

    Character Read() noexcept;

    int Read(CharacterArray &array) noexcept;
    int ReadLine(CharacterArray &array) noexcept;

    void PrintInfo() const noexcept;


    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      終了判定
     */
    /* ------------------------------------------------------------------------- */
    constexpr bool IsTerminated() const noexcept
    {
        return (_offset >= _size);
    }

private:
    TextEncoding CheckEncoding(bool &existBOM) const noexcept;
    size_t GetStartOffset() const noexcept;

    Character ReadUTF8() noexcept;
    Character ReadUTF16() noexcept;
    Character ReadUTF32() noexcept;

    const uint8_t *_buffer;
    size_t _size;
    TextEncoding _encoding;
    size_t _offset;
    bool _existBOM;
    std::unique_ptr<uint8_t[]> _loadBuffer;
};
}   // namespace MTK


#endif  // INCGUARD_TEXT_STREAM_H_1538586799

// vim: et ts=4 sw=4 sts=4
