// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       text_format.cc
 *  \brief      テキストフォーマット
 *  \date       Since: June 28, 2021. 10:31:37 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mtk/text_format.h>

namespace MTK
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      文字列のフォーマット
 *  \param[in]  text    変換元の文字列
 *  \param[in]  args    引数
 *  \return     変換後の文字列
 */
/* ------------------------------------------------------------------------- */
std::string Format(const std::string &text, const FormatArgs &args) noexcept
{
    std::string dest;

    const char *source = text.c_str();
    FormatArgument::Option option;
    size_t autoIndex = 0;

    while (*source != '\0')
    {
        // 現在の先頭からパースして有効であれば対応した引数に置き換える
        source = FormatArgument::Parse(option, source);
        if (option.isValid)
        {
            // インデックスの指定があればそのインデックス，無ければ自動インデックスを使う
            size_t argsIndex = autoIndex;
            if (option.index >= 0)
            {
                argsIndex = static_cast<size_t>(option.index);
            }
            
            // インデックスが引数の有効範囲内であれば置き換える
            if (argsIndex < args.size())
            {
                dest += args[argsIndex].ToString(option);
            }
            // 有効範囲外
            else
            {
                dest += "<!OOR!>";  // Out Of Range
            }
            
            // 自動インデックスをスキップするオプションが指定されていなければ更新
            if (!option.skipAutoIndex)
            {
                autoIndex++;
            }
        }
        // 置き換え対象でなければ出力先に文字を追加
        else
        {
            dest += *source;
            source++;
        }
    }

    return dest;
}
}   // namespace MTK

// vim: et ts=4 sw=4 sts=4
