/* ------------------------------------------------------------------------- */
/*!
 *  \file       rectangle.cc
 *  \brief      矩形と矩形リスト
 *  \date       Since: June 7, 2021. 17:12:14 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include "rectangle.h"
#include <algorithm>

namespace texatlas
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      矩形の分割
 *  \param[in]  rectangle   使用する矩形
 *  \return     分割後の矩形
 */
/* ------------------------------------------------------------------------- */
std::vector<Rectangle> Rectangle::Split(const Rectangle &rectangle) noexcept
{
    std::vector<Rectangle> array;

    array.push_back({x + rectangle.width, y, width - rectangle.width, rectangle.height});
    array.push_back({x, y + rectangle.height, width, height - rectangle.height});

    return array;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      丁度良いサイズの矩形を取得する
 *  \param[in]  rectangle   元になる矩形
 *  \return     引数で受けた矩形が丁度良く収まる矩形
 */
/* ------------------------------------------------------------------------- */
Rectangle RectangleList::PopFitRectangle(const Rectangle &rectangle) noexcept
{
    auto it = std::find_if(begin(), end(), [rectangle](const Rectangle &elem)
    {
        if (elem.height >= rectangle.height)
        {
            if (elem.width >= rectangle.width)
            {
                return true;
            }
        }

        return false;
    });


    if (it != end())
    {
        Rectangle result(*it);
        erase(it);
        return result;
    }

    return Rectangle();
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      矩形を追加
 *  \param[in]  rectangle   追加する矩形
 */
/* ------------------------------------------------------------------------- */
void RectangleList::Add(const Rectangle &rectangle) noexcept
{
    auto it = std::lower_bound(begin(), end(), rectangle, [](const Rectangle &rect1, const Rectangle &rect2)
    {
        return rect1.GetSortKey() < rect2.GetSortKey();
    });

    if ((rectangle.width != 0) && (rectangle.height != 0))
    {
        insert(it, rectangle);
    }
}
}   // namespace texatlas

// vim: et ts=4 sw=4 sts=4
