/* ------------------------------------------------------------------------- */
/*!
 *  \file       rectangle.h
 *  \brief      矩形と矩形リスト
 *  \date       Since: June 7, 2021. 17:12:12 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_RECTANGLE_H_1623053532
#define INCGUARD_RECTANGLE_H_1623053532

#include <cstdint>
#include <list>
#include <vector>

namespace texatlas
{
//! 矩形クラス
struct Rectangle
{
    uint32_t x;
    uint32_t y;
    uint32_t width;
    uint32_t height;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    constexpr Rectangle() noexcept
        : x(0)
        , y(0)
        , width(0)
        , height(0)
    {}
        
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  inX         X座標
     *  \param[in]  inY         Y座標
     *  \param[in]  inWidth     幅
     *  \param[in]  inHeight    高さ
     */
    /* ------------------------------------------------------------------------- */
    constexpr Rectangle(uint32_t inX, uint32_t inY, uint32_t inWidth, uint32_t inHeight) noexcept
        : x(inX)
        , y(inY)
        , width(inWidth)
        , height(inHeight)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      boolへのキャスト
     *  \retval     true    このクラスが有効
     *  \retval     false   このクラスは無効
     */
    /* ------------------------------------------------------------------------- */
    explicit constexpr operator bool() const noexcept
    {
        return (width > 0) && (height > 0);
    }
    
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      否定演算子
     *  \retval     true    このクラスは無効
     *  \retval     false   このクラスが有効
     */
    /* ------------------------------------------------------------------------- */
    constexpr bool operator!() const noexcept
    {
        return !static_cast<bool>(*this);
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ソートに使用する値を取得
     *  \return     ソートに使用する値
     */
    /* ------------------------------------------------------------------------- */
    constexpr uint64_t GetSortKey() const noexcept
    {
        return (static_cast<uint64_t>(height) << 32) | width;
    }

    std::vector<Rectangle> Split(const Rectangle &rectangle) noexcept;
};


//! 矩形リストクラス
class RectangleList : public std::list<Rectangle>
{
public:
    Rectangle PopFitRectangle(const Rectangle &rectangle) noexcept;

    void Add(const Rectangle &rectangle) noexcept;
};

}   // namespace texatlas

#endif	// INCGUARD_RECTANGLE_H_1623053532

// vim: et ts=4 sw=4 sts=4
